/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** digitally-imported-ui
**
** an UbuntuTouch frontend for DigitallyImported's music-service
**
** Copyright 2013 Mirco "MacSlow" Müller <macslow@gmail.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

import QtQuick 2.0
import Ubuntu.Components 0.1

Item {
    id: channel

    property alias icon: iconImage.source
    property alias title: titleLabel.text
    property alias description: descriptionLabel.text
    property alias track: trackLabel.text
    property var player
    property var view
    property var textColor

    width: childrenRect.width + units.gu(1)
    height: childrenRect.height + units.gu(1)

    MouseArea {
        anchors.fill: contentRow
        onClicked: {
            if (view.currentIndex == -1) {
                view.currentIndex = index
                view.currentItem.textColor = "white"
                player.play(uri)
            } else if (view.currentIndex == index) {
                view.currentItem.textColor = Qt.darker("#aea79f", 1.3);
                view.currentIndex = -1
                player.stop()
            } else {
                view.currentItem.textColor = Qt.darker("#aea79f", 1.3);
                view.currentIndex = index
                view.currentItem.textColor = "white"
                player.crossfade(uri)
            }
        }
    }

    Row {
        id: contentRow

        spacing: units.gu(1)
        x: units.gu(.5) // this is a...
        y: units.gu(.5) // ...bit ugly

        UbuntuShape {
            width: units.gu(7)
            height: units.gu(7)

            image: Image {
                id: iconImage
            }
        }

        Column {
            Label {
                id: titleLabel

                font.bold: true
                color: textColor
                maximumLineCount: 1
                elide: Text.ElideRight
                Behavior on color { ColorAnimation { duration: 500 } }
            }

            Label {
                id: descriptionLabel

                width: units.gu(30)
                color: textColor
                maximumLineCount: 2
                wrapMode: Text.WordWrap
                elide: Text.ElideRight
                fontSize: "small"
                Behavior on color { ColorAnimation { duration: 500 } }
            }

            Label {
                id: trackLabel

                width: units.gu(30)
                maximumLineCount: 1
                color: textColor
                elide: Text.ElideRight
                fontSize: "xx-small"
                Behavior on color { ColorAnimation { duration: 500 } }
            }
        }
    }
}
