/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** gst-crossfade
**
** crossfade between two pipeline playing back shoutcast/icecast streams
**
** Copyright 2013 Mirco "MacSlow" Müller <macslow@gmail.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

/*
** how to compile:
**   gcc -ogst-crossfade-controller gst-crossfade-controller.c `pkg-config --cflags --libs gstreamer-0.10 gstreamer-controller-0.10`
*/

/*
** how to mix two audio-streams...
**
**   gst-launch
**      souphttpsrc location=http://pub4.di.fm:80/di_ukgarage_aacplus !
**      decodebin name=decode0 ! volume name=vol0 volume=0.01 ! adder name=mix !
**      audioconvert ! alsasink
**      souphttpsrc location=http://pub4.di.fm:80/di_techhouse_aacplus !
**      decodebin name=decode1 ! volume name=vol1 volume=0.2 ! mix.
*/

#include <glib.h>
#include <gst/gst.h>
#include <gst/controller/gstcontroller.h>

void on_pad_added (GstElement* decode,
				   GstPad*     pad,
				   GstElement* volume)
{
	GstPad* volumepad = NULL;

	/* only link once */
	volumepad = gst_element_get_static_pad (volume, "sink");
	if (GST_PAD_IS_LINKED (volumepad))
	{
		g_object_unref (volumepad);
		return;
	}

	/* link'n'play */
	gst_pad_link (pad, volumepad);

	g_object_unref (volumepad);
}

int main (int argc, char** argv)
{
	GMainLoop*  loop      = NULL;
	GstElement* pipeline  = NULL;
	GstElement* source0   = NULL;
	GstElement* source1   = NULL;
	GstElement* decode0   = NULL;
	GstElement* decode1   = NULL;
	GstElement* volume0   = NULL;
	GstElement* volume1   = NULL;
	GstElement* mixer     = NULL;
	GstElement* convert   = NULL;
	GstElement* sink      = NULL;

	gst_init (&argc, &argv);
	gst_controller_init (&argc, &argv);

	loop = g_main_loop_new (NULL, FALSE);

	pipeline = gst_pipeline_new ("pipeline");
	source0  = gst_element_factory_make ("souphttpsrc",  "source0");
	source1  = gst_element_factory_make ("souphttpsrc",  "source1");
	decode0  = gst_element_factory_make ("decodebin",    "decode0");
	decode1  = gst_element_factory_make ("decodebin",    "decode1");
	volume0  = gst_element_factory_make ("volume",       "volume0");
	volume1  = gst_element_factory_make ("volume",       "volume1");
	mixer    = gst_element_factory_make ("adder",        "mixer");
	convert  = gst_element_factory_make ("audioconvert", "convert");
	sink     = gst_element_factory_make ("alsasink",     "sink");

	g_object_set (G_OBJECT (source0),
				  "location",
				  "http://pub4.di.fm:80/di_techhouse_aacplus",
				  NULL);
	g_object_set (G_OBJECT (source1),
				  "location",
				  "http://pub4.di.fm:80/di_ukgarage_aacplus",
				  NULL);

	g_object_set (G_OBJECT (volume0), "volume", 0.0, NULL);
	g_object_set (G_OBJECT (volume1), "volume", 0.0, NULL);

	g_signal_connect (decode0, "pad-added", G_CALLBACK(on_pad_added), volume0);
	g_signal_connect (decode1, "pad-added", G_CALLBACK(on_pad_added), volume1);
	gst_bin_add_many (GST_BIN (pipeline),
					  source0,
					  source1,
					  decode0,
					  decode1,
					  volume0,
					  volume1,
					  mixer,
					  convert,
					  sink,
					  NULL);
	gst_element_link_pads(source0, "src", decode0, "sink");
	gst_element_link_pads(source1, "src", decode1, "sink");
	gst_element_link_pads(volume0, "src", mixer, "sink0");
	gst_element_link_pads(volume1, "src", mixer, "sink1");
	gst_element_link_pads(mixer, "src", convert, "sink");
	gst_element_link_pads(convert, "src", sink, "sink");

    GstController* controller0 = gst_controller_new (G_OBJECT (volume0),
													 "volume",
													 NULL);
    GstController* controller1 = gst_controller_new (G_OBJECT (volume1),
													 "volume",
													 NULL);

	gst_object_control_properties (G_OBJECT (volume0), "volume", NULL);
	gst_object_control_properties (G_OBJECT (volume1), "volume", NULL);
	GstInterpolationControlSource* csource0 = gst_interpolation_control_source_new();
	GstInterpolationControlSource* csource1 = gst_interpolation_control_source_new();
	gst_controller_set_control_source (controller0,
									   "volume",
									   GST_CONTROL_SOURCE(csource0));
	gst_controller_set_control_source (controller1,
									   "volume",
									   GST_CONTROL_SOURCE(csource1));
	gst_interpolation_control_source_set_interpolation_mode (csource0,
															 GST_INTERPOLATE_LINEAR);
	gst_interpolation_control_source_set_interpolation_mode (csource1,
															 GST_INTERPOLATE_LINEAR);
	GValue* value0 = g_new0(GValue, 1);
	GValue* value1 = g_new0(GValue, 1);
	value0 = g_value_init (value0, G_TYPE_DOUBLE);
	value1 = g_value_init (value1, G_TYPE_DOUBLE);
	g_value_set_double (value0, 0.0);
	g_value_set_double (value1, 1.0);
	gst_interpolation_control_source_set (csource0, 0 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource0, 0.5 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource0, 3 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource0, 3.5 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource1, 0 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource1, 4 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource1, 4.5 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource1, 7 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource1, 7.5 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource0, 8 * GST_SECOND, value0);
	gst_interpolation_control_source_set (csource0, 8.5 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource0, 11 * GST_SECOND, value1);
	gst_interpolation_control_source_set (csource0, 11.5 * GST_SECOND, value0);

	gst_element_set_state (pipeline, GST_STATE_PLAYING);

	g_main_loop_run (loop);

	gst_object_unref (controller0);
	gst_object_unref (controller1);
	gst_object_unref (csource0);
	gst_object_unref (csource1);
	g_free (value0);
	g_free (value1);
	gst_element_set_state (pipeline, GST_STATE_NULL);
	gst_object_unref (GST_OBJECT (pipeline));
	g_main_loop_unref(loop);

	return 0;
}
