/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** digitally-imported-ui
**
** an UbuntuTouch frontend for DigitallyImported's music-service
**
** Copyright 2013 Mirco "MacSlow" Müller <macslow@gmail.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include <glib.h>

#include "player.h"

void handleTag (const GstTagList* list,
                const gchar*      tag,
                gpointer          data)
{
	if (!strcmp(GST_TAG_TITLE, tag))
	{
		gchar* title = NULL;
		gst_tag_list_get_string (list, GST_TAG_TITLE, &title);
		((Player*)data)->setCurrentTitle (QString(title));
		g_free(title);
	}
}

void gotMessageTag (GstBus*     bus,
                    GstMessage* message,
                    Player*   data)
{
    GstTagList* tags = NULL;
    gst_message_parse_tag (message, &tags);
    gst_tag_list_foreach (tags, handleTag, data);
    gst_tag_list_free (tags);
}

Player::Player (QObject* parent) : QObject (parent)
{
	_playbin2 = gst_element_factory_make ("playbin2", "playbin2");
	_bus = gst_element_get_bus (_playbin2);
	gst_bus_add_signal_watch(_bus);
	g_signal_connect (_bus, "message::tag", G_CALLBACK(gotMessageTag), this);
	gst_object_unref(_bus);
    _title = "";
}

Player::~Player ()
{
	gst_object_unref (_bus);
	gst_element_set_state (_playbin2, GST_STATE_NULL);
	gst_object_unref (_playbin2);
}

void Player::stop ()
{
	gst_element_set_state (_playbin2, GST_STATE_NULL);
}

void Player::play (QString uri)
{
	gst_element_set_state (_playbin2, GST_STATE_NULL);
	g_object_set (_playbin2, "uri", uri.toStdString().c_str(), NULL);
	gst_element_set_state (_playbin2, GST_STATE_PLAYING);
}

void Player::crossfade (QString uri)
{
	//
	// how to mix two audio-streams...
	//
	//   gst-launch
	//      souphttpsrc location=http://pub4.di.fm:80/di_ukgarage_aacplus?a2f2e8f763ba8b6a186d3e0f !
	//      decodebin name=decode0 ! volume name=vol0 volume=0.01 ! adder name=mix !
	//      audioconvert ! alsasink
	//      souphttpsrc location=http://pub4.di.fm:80/di_techhouse_aacplus?a2f2e8f763ba8b6a186d3e0f !
	//      decodebin name=decode1 ! volume name=vol1 volume=0.2 ! mix.
	//

	play (uri);
}

void Player::setCurrentTitle (QString title)
{
	_title = title;
	emit titleChanged();
}

QString Player::getCurrentTitle ()
{
	return _title;
}

#include "moc_player.h"